package org.signwriting;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * reads the symbol file
 */
public class KeyFile {

    public final static int Z     = 0x2C;
    public final static int X     = 0x2D;
    public final static int C     = 0x2E;
    public final static int V     = 0x2F;
    public final static int B     = 0x30;
    public final static int N     = 0x31;
    public final static int M     = 0x32;
    public final static int Comma = 0x33;
    public final static int Dot   = 0x34;
    public final static int Slash = 0x35;

    public final static int A     = 0x1E;
    public final static int S     = 0x1F;
    public final static int D     = 0x20;
    public final static int F     = 0x21;
    public final static int G     = 0x22;
    public final static int H     = 0x23;
    public final static int J     = 0x24;
    public final static int K     = 0x25;
    public final static int L     = 0x26;
    public final static int Semi  = 0x27;
    public final static int Rquot = 0x28;

    public final static int Q     = 0x10;
    public final static int W     = 0x11;
    public final static int E     = 0x12;
    public final static int R     = 0x13;
    public final static int T     = 0x14;
    public final static int Y     = 0x15;
    public final static int U     = 0x16;
    public final static int I     = 0x17;
    public final static int O     = 0x18;
    public final static int P     = 0x19;
    public final static int Lbrac = 0x1A;
    public final static int Rbrac = 0x1B;
    public final static int Baksl = 0x2B;

    public final static int Lquot = 0x29;
    public final static int n1    = 0x02;
    public final static int n2    = 0x03;
    public final static int n3    = 0x04;
    public final static int n4    = 0x05;
    public final static int n5    = 0x06;
    public final static int n6    = 0x07;
    public final static int n7    = 0x08;
    public final static int n8    = 0x09;
    public final static int n9    = 0x0A;
    public final static int n0    = 0x0B;
    public final static int Minus = 0x0C;
    public final static int Equal = 0x0D;

    public final static int zbs  = 0x0E;// debug -- command key scan codes
    public final static int zret = 0x1C;
    public final static int zesc = 0x01;
    public final static int ztab = 0x0F;
    
    private final static int SIZE_BIAS = 0;
    // arrow heads
    private final static int SOLID = 1;
    private final static int HOLLOW = 2;
    private final static int OPEN = 3;

    private final static int MAX_PAIR = 4;
    private final static int MAX_D = 100;
    private final static int K_MAX_MAP = 3000;

    public final static int SIGN_TYPE = 0;
    public final static int FINGER_TYPE = 1;
    public final static int ALPHA_TYPE = 2;
    private final static int TYPE_MODE_COUNT = 3;

    private final static int SCAN_CODE_COUNT = 256;

    public final static int NO_MODIFIER = 0;
    public final static int SHIFT_MODIFIER = 1;
    public final static int ALT_MODIFIER = 2;
    public final static int CTRL_MODIFIER = 3;
    public final static int CTRL_SHIFT_MODIFIER = 4;
    public final static int CTRL_ALT_MODIFIER = 5;
    public final static int SHIFT_ALT_MODIFIER = 6;
    private final static int MODIFIER_COUNT = 7;

    String filename;
    static InxOrSymRec[][][] keyboard;
    static byte keymap[];
    
    // fingerspelling symbol, option, and descent
    static byte fsym[];
    static byte fopt[];
    static byte dval[];

    /**
     * constructor, takes a symbol file name, opens it and reads it
     */
    public KeyFile() {
        filename = Resources.getResources().getString("KeyFile.file");
    }

    public void readKeys() throws IOException {

        DataInputStream in = new DataInputStream(
                             new BufferedInputStream(
                             new FileInputStream(filename)));

		int oldAvail = in.available();

		int fHandCode = in.read(); // fingerspell icon sym code
		int fHandOpt = in.read(); // fingerspell opt byte
		int partCode = in.read(); // "Part" icon codes

		// read in the entire sort table
		// (or we can skip the next 511 bytes)

        int country = PCNumbers.makeShort(in.read(),in.read());
		byte[] charMap = new byte[256];
		for (int i = 0; i < 256; i++) {
		    charMap[i] = in.readByte(); // 256 bytes
		}
		PairRec[] pairs = new PairRec[MAX_PAIR+1];
		for (int i = 0; i <= MAX_PAIR; i++) {
		    pairs[i] = new PairRec();
		    pairs[i].read(in); // 30 bytes
		}

		// ctrl-cursor codes
		int upCode = in.read();
		int downCode = in.read();
		int leftCode = in.read();
		int rightCode = in.read();
		int quickCode = in.read();

		int numRead = oldAvail - in.available();

		if ((512 - numRead) != 216) {
		    throw new IOException("Expected to be 216 bytes from header but am "+(512-numRead));
		}
		in.skip(512 - numRead); // skip header

/** READ KEYBOARD STRUCTURE **/

        keyboard = new InxOrSymRec[TYPE_MODE_COUNT][][];
        for (int i = 0; i < TYPE_MODE_COUNT; i++) {
            keyboard[i] = new InxOrSymRec[SCAN_CODE_COUNT][];
            for (int j = 0; j < SCAN_CODE_COUNT; j++) {
                keyboard[i][j] = new InxOrSymRec[MODIFIER_COUNT];
                for (int k = 0; k < MODIFIER_COUNT; k++) {
                    int codeval = in.read();
                    int rotval = in.read();
                    if (rotval <= 127) {
                        keyboard[i][j][k] = new InxRec(PCNumbers.makeShort(codeval,rotval));
                    } else {
                        keyboard[i][j][k] = new SymRec(codeval,rotval);
                    }
                }
            }
        }

/** READ DESCENDER STRUCTURE **/

        fsym = new byte[MAX_D+1];  // target symbol
        for (int i = 0; i <= MAX_D; i++) {
            fsym[i] = in.readByte();
        }
        in.read(); // read filler byte

        fopt = new byte[MAX_D+1];
        for (int i = 0; i <= MAX_D; i++) {
            fopt[i] = in.readByte();
        }
        in.read(); // read filler byte

        dval = new byte[MAX_D+1];  // descent value
        for (int i = 0; i <= MAX_D; i++) {
            dval[i] = in.readByte();
        }
        in.read(); // read filler byte

/** READ KEYMAP STRUCTURE **/
        keymap = new byte[K_MAX_MAP+1];  // descent value
        for (int i = 0; i <= K_MAX_MAP; i++) {
            keymap[i] = in.readByte();
        }
        in.read(); // read filler byte
        in.close();
    }
    
    public static int getScanCode(char keycode) {
        switch (keycode) {
            case '`': return Lquot;
            case '1': return n1;
            case '2': return n2;
            case '3': return n3;
            case '4': return n4;
            case '5': return n5;
            case '6': return n6;
            case '7': return n7;
            case '8': return n8;
            case '9': return n9;
            case '0': return n0;
            case '-': return Minus;
            case '=': return Equal;

            case '~': return Lquot;
            case '!': return n1;
            case '@': return n2;
            case '#': return n3;
            case '$': return n4;
            case '%': return n5;
            case '^': return n6;
            case '&': return n7;
            case '*': return n8;
            case '(': return n9;
            case ')': return n9;
            case '_': return Minus;
            case '+': return Equal;

            case 'q': return Q;
            case 'w': return W;
            case 'e': return E;
            case 'r': return R;
            case 't': return T;
            case 'y': return Y;
            case 'u': return U;
            case 'i': return I;
            case 'o': return O;
            case 'p': return P;
            case '[': return Lbrac;
            case ']': return Rbrac;
            case '\\': return Baksl;

            case 'Q': return Q;
            case 'W': return W;
            case 'E': return E;
            case 'R': return R;
            case 'T': return T;
            case 'Y': return Y;
            case 'U': return U;
            case 'I': return I;
            case 'O': return O;
            case 'P': return P;
            case '{': return Lbrac;
            case '}': return Rbrac;
            case '|': return Baksl;

            case 'a': return A;
            case 's': return S;
            case 'd': return D;
            case 'f': return F;
            case 'g': return G;
            case 'h': return H;
            case 'j': return J;
            case 'k': return K;
            case 'l': return L;
            case ';': return Semi;
            case '\'': return Rquot;

            case 'A': return A;
            case 'S': return S;
            case 'D': return D;
            case 'F': return F;
            case 'G': return G;
            case 'H': return H;
            case 'J': return J;
            case 'K': return K;
            case 'L': return L;
            case ':': return Semi;
            case '\"': return Rquot;

            case 'z': return Z;
            case 'x': return X;
            case 'c': return C;
            case 'v': return V;
            case 'b': return B;
            case 'n': return N;
            case 'm': return M;
            case ',': return Comma;
            case '.': return Dot;
            case '/': return Slash;

            case 'Z': return Z;
            case 'X': return X;
            case 'C': return C;
            case 'V': return V;
            case 'B': return B;
            case 'N': return N;
            case 'M': return M;
            case '<': return Comma;
            case '>': return Dot;
            case '?': return Slash;

            case 8 : return zbs;
            case 13: return zret;
            case 27: return zesc;
            case 9 : return ztab;

            default: return 255;
        }
    }
    
    public static int getKeynum(int scanCode) {
	    int keynum = -1;
	    switch(scanCode) {
	        case A:
	            keynum = 0;
	            break;
	        case S:
	            keynum = 1;
	            break;
	        case D:
	            keynum = 2;
	            break;
	        case F:
	            keynum = 3;
	            break;
	        case J:
	            keynum = 4;
	            break;
	        case K:
	            keynum = 5;
	            break;
	        case L:
	            keynum = 6;
	            break;
	        case Semi:
	            keynum = 7;
	            break;
	        case G:
	            keynum = 8;
	            break;
	        case H:
	            keynum = 9;
	            break;
	    }
	    return keynum;
	}
	
	
    public static int keyToOpt(int val) {
        byte opt = 8; //StockOpt;
        int size = 0;

        while (val >= 144) {
            val -= 16;
            size++;
        }

        opt = writeSize(opt, size);

        if (val >= 136) {
            opt |= 1 << 7; // set flop bit
            val -= 8;
        }

        int rot = val - 128;
        opt |= rot;
        return PCNumbers.toUnsigned(opt);
    }
    
    private static byte writeAhead(byte opt, int val) {
        val &= 0x3;         // Get 2 lower bits
        val <<= 3;           // Shift left 3
        opt &= 0xe7;       // Clear 4th and 5th bits
        opt |= val;        // 2 bits from val now in 4th & 5th bits
        return opt;
    }

    private static byte readAhead(byte opt) {
        byte x = (byte)(opt >>> 3);           // Right justify 4th bit
        return (byte)(x & 0x3);           // Set val to lower 2 bits of Ahead field
    }

    private static byte writeSize(byte opt, int val) {
        // Put value into Size field (6th and 7th bits) of optbyte.  If too
        // large to fit, decrease Size value and set Ahead field = SizeBias.

        byte ahd = readAhead(opt); // set Ahead to reflect Size value

        if (val >= 4) {
            opt = writeAhead(opt, SIZE_BIAS);
            val -= 4;
        } else if (ahd == SIZE_BIAS) {
            opt = writeAhead(opt, SOLID);
        }

        byte x = (byte)(val & 0x3);      // Get 2 lower bits
        x <<= 5;              // Shift left 5
        opt &= 0x9f;       // Clear 6th and 7th bits
        opt |= x;          // 2 bits from val now in 6th & 7th bits
        return opt;
    }
}


